// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a BSD 2-Clause "Simplified" license
// that can be found in the LICENSE file.

library linear_structures;

import 'singly_linked_list.dart';

/// Complexity of each operation
/// ----------------------------
///  operation | complexity
///  size      |   O(1)
///  isEmpty   |   O(1)
///  peek      |   O(1)
///  enqueue   |   O(1)
///  dequeue   |   O(1)

/// Implementation of Queue using a SinglyLinkedList
class Queue<T> {
  int _rear = -1;
  SinglyLinkedList<T> _elements = SinglyLinkedList<T>();

  /// Returns the size of the queue.
  int get size => _rear + 1;

  /// Returns true if the queue is empty.
  bool get isEmpty => _rear == -1;

  /// Returns front element in the queue.
  /// This method returns null if the queue is empty.
  T? peek() {
    if (isEmpty) return null;
    return _elements.peekFirst;
  }

  /// Adds an element to the queue.
  void enqueue(T obj) {
    _elements.addLast(obj);
    _rear++;
  }

  /// Removes front element from the queue.
  /// This method returns the dequeued element
  /// or returns null if the queue is empty.
  T? dequeue() {
    if (isEmpty) return null;
    T? dequeuedElement = _elements.removeFirst();
    _rear--;
    return dequeuedElement;
  }
}
