// Copyright 2020 Ammar Yasser. All rights reserved.
// Use of this source code is governed by a BSD 2-Clause "Simplified" license
// that can be found in the LICENSE file.

library linear_structures;

import 'singly_linked_list.dart';

/// Complexity of each operation
/// ----------------------------
///  operation | complexity
///  size      |   O(1)
///  isEmpty   |   O(1)
///  peek      |   O(1)
///  push      |   O(1)
///  pop       |   O(n)

/// Implementation of Stack using a SinglyLinkedList
class Stack<T> {
  int _top = -1;
  SinglyLinkedList<T> _elements = SinglyLinkedList<T>();

  /// Returns the size of the stack.
  int get size => _top + 1;

  /// Returns true if the stack is empty.
  bool get isEmpty => _top == -1;

  /// Displays the element on the top of the stack.
  /// This method returns null if the stack is empty.
  T? peek() {
    if (isEmpty) return null;
    return _elements.peekLast;
  }

  /// Pushes an element to the top of the stack.
  void push(T obj) {
    _elements.addLast(obj);
    _top++;
  }

  /// Pops an element from the top of the stack.
  /// This method returns the popped element
  /// or returns null if the stack is empty.
  T? pop() {
    if (isEmpty) return null;
    T? removedElement = _elements.removeLast();
    _top--;
    return removedElement;
  }
}
